<?php

/**
 * Page callback for the add block page.
 */
function felix_add_block_page() {
  if (empty($_GET['region']) || empty($_GET['path'])) {
    return MENU_NOT_FOUND;
  }
  
  if (!$region = _felix_region_by_name($_GET['region'])) {
    return MENU_NOT_FOUND;
  }
  
  if (!empty($_GET['nodetype'])) {
    $salt = $_GET['nodetype'] . $region->name;
    if (drupal_get_token($salt) != $_GET['token']) {
      return MENU_ACCESS_DENIED;
    }
    return _felix_add_block_nodetype_page($region, $_GET['nodetype'], $_GET['path']);
  }
  elseif (!empty($_GET['module']) && !empty($_GET['delta'])) {
    $salt = $_GET['module'] . $_GET['delta'] . $region->name;
    if (drupal_get_token($salt) != $_GET['token']) {
      return MENU_ACCESS_DENIED;
    }
    return _felix_add_block($region, $_GET['path'], $_GET['module'], $_GET['delta']);
  }
  elseif (!empty($_GET['nid'])) {
    $salt = $_GET['nid'] . $region->name;
    if (drupal_get_token($salt) != $_GET['token']) {
      return MENU_ACCESS_DENIED;
    }
    return _felix_add_block($region, $_GET['path'], 'node', $_GET['nid'], 'node/' . $_GET['nid']);
  }
  else {
    // Show the list of possible blocks and nodetypes.
    return _felix_add_block_list_page($region, $_GET['path']);
  }
}

function _felix_add_block_nodetype_page($region, $nodetype, $path) {
  $output = '';
  
  $output .= '<h2>' . t('Search node') . '</h2>';
  $output .= drupal_render(drupal_get_form('felix_add_node_form', $region, $nodetype, $path));
  
  $output .= '<h2>' . t('Recent content') . '</h2>';
  $nodes = db_select('node', 'n')
          ->fields('n', array('nid', 'title'))
          ->condition('n.type', $nodetype)
          ->orderBy('n.changed', 'DESC')
          ->range(0, 10)
          ->execute();
  $items = array();
  while ($node = $nodes->fetchObject()) {
    $items[] = array(
      'title' => $node->title,
      'href' => 'felix-blocks/add',
      'query' => array(
        'nid' => $node->nid,
        'region' => $region->name,
        'path' => $path,
        'token' => drupal_get_token($node->nid . $region->name),
      ),
    );
  }
  if ($items) {
    $output .= theme('links', array('links' => $items));
  }
  else {
    $output .= '<p>' . t('No content found.') . '</p>';
  }
  
  return $output;
}

function felix_add_node_form($form, &$form_state, $region, $nodetype, $path) {
  $form['#felix_region'] = $region;
  $form['#felix_nodetype'] = $nodetype;
  $form['#felix_path'] = $path;
  
  $form['node'] = array(
    '#type' => 'textfield',
    '#title' => t('Title'),
    '#autocomplete_path' => 'felix-blocks/autocomplete-node/' . $nodetype,
    '#required' => TRUE,
  );
  
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Add'),
  );
  
  return $form;
}

function felix_add_node_form_validate($form, &$form_state) {
  if (!preg_match('/\\(\\#([0-9]+)\\)$/s', $form_state['values']['node'], $match)) {
    form_set_error('node', t('Invalid format'));
  }
  else {
    $nodetype = db_select('node', 'n')
            ->fields('n', array('type'))
            ->condition('n.nid', $match[1])
            ->execute()
            ->fetchField();
    if ($nodetype !== $form['#felix_nodetype']) {
      form_set_error('node', t('Node not found'));
    }
  }
}

function felix_add_node_form_submit($form, &$form_state) {
  preg_match('/\\(\\#([0-9]+)\\)$/s', $form_state['values']['node'], $match);
  $nid = $match[1];
  _felix_add_block($form['#felix_region'], $form['#felix_path'], 'node', $nid, "node/$nid");
}

function _felix_add_block($region, $path, $module, $delta, $detail_path = NULL, $data = array()) {
  $block = new stdClass();
  $block->region = $region->name;
  $block->weight = _felix_new_block_weight($region->name);
  $block->hash = _felix_get_hash($region, $path);
  $block->module = $module;
  $block->delta = $delta;
  $block->detail_path = $detail_path;
  $block->data = $data;
  
  foreach (module_implements('felix_block_insert') as $module) {
    $hook = "{$module}_felix_block_insert";
    $hook(&$block, $path);
  }
  
  $block->data = serialize($block->data);
  
  $exists = db_select('felix_block', 'cb')
          ->fields('cb', array('region'))
          ->condition('cb.region', $block->region)
          ->condition('cb.hash', $block->hash)
          ->condition('cb.module', $block->module)
          ->condition('cb.delta', $block->delta)
          ->countQuery()
          ->execute()
          ->fetchField();
  if ($exists) {
    drupal_set_message(t('This block already exists in this region.'), 'warning');
  }
  else {
    drupal_write_record('felix_block', $block);
    drupal_set_message(t('Succesfully added a new block to the region %region.', array('%region' => $region->title)));
  }
  
  // Expire external cache.
  if (module_exists('expire_cache_derivative')) {
    expire_cache_derivative(array($path));
  }
  
  drupal_goto($path);
}

function _felix_add_block_list_page($region, $path) {
  // Build a list of all possible items (blocks or nodetypes).
  $items = array();
  
  // Add list of nodetypes.
  $nodetypes = db_select('felix_block_set_nodetype', 'nt')
          ->fields('nt', array('nodetype'))
          ->orderBy('nt.nodetype')
          ->execute()
          ->fetchCol();
  if ($nodetypes) {
    $nodetype_names = node_type_get_names();
    $items['_nodetypes'] = array();
    foreach ($nodetypes as $nodetype) {
      $items['_nodetypes'][] = array(
        'title' => $nodetype_names[$nodetype],
        'href' => 'felix-blocks/add',
        'query' => array(
          'nodetype' => $nodetype,
          'region' => $region->name,
          'path' => $path,
          'token' => drupal_get_token($nodetype . $region->name),
        ),
      );
    }
  }
  
  // Add blocks.
  $blocks = db_select('felix_block_set_block', 'b')
          ->fields('b', array('module', 'delta'))
          ->condition('b.block_set', $region->block_set)
          ->orderBy('b.module')
          ->execute()
          ->fetchAll();
  foreach ($blocks as $block) {
    $info = module_invoke($block->module, 'block_info');
    if (!isset($info[$block->delta])) {
      continue;
    }
    if (!isset($items[$block->module])) {
      $items[$block->module] = array();
    }
    $items[$block->module][] = array(
      'title' => $info[$block->delta]['info'],
      'href' => 'felix-blocks/add',
      'query' => array(
        'module' => $block->module,
        'delta' => $block->delta,
        'region' => $region->name,
        'path' => $path,
        'token' => drupal_get_token($block->module . $block->delta . $region->name),
      ),
    );
  }
  
  $output = '';
  foreach ($items as $name => $links) {
    if ($name == '_nodetypes') {
      $name = t('Nodetype');
    }
    else {
      $info = system_get_info('module', $name);
      $name = $info['name'];
    }
    
    $output .= '<h2>' . check_plain($name) . '</h2>';
    $output .= theme('links', array('links' => $links));
  }
  return $output;
}

function felix_autocomplete_node($nodetype, $text) {
  $matches = array();
  $nodes = db_select('node', 'n')
    ->fields('n', array('nid', 'title'))
    ->condition('n.type', $nodetype)
    ->condition('n.title', db_like($text) . '%', 'LIKE')
    ->orderBy('n.changed', 'DESC')
    ->range(0, 10)
    ->execute()
    ->fetchAll();
  foreach ($nodes as $node) {
    $matches["{$node->title} (#{$node->nid})"] = check_plain($node->title);
  }
  drupal_json_output($matches);
}

/**
 * Page callback for block actions.
 * 
 * This function is only accessible with the following GET parameters:
 * 
 * - region
 * - path
 * - module
 * - delta
 * - hash
 * - token => drupal_get_token(region module delta hash)
 * 
 * @param string $action One of 'move-up', 'move-down', 'attributes' or 'remove'
 */
function felix_block_action($action) {
  if (empty($_GET['region'])
    || empty($_GET['path'])
    || empty($_GET['module'])
    || empty($_GET['delta'])
    || !isset($_GET['hash'])
    || empty($_GET['token'])) {
    return MENU_NOT_FOUND;
  }
  
  if (!$region = _felix_region_by_name($_GET['region'])) {
    return MENU_NOT_FOUND;
  }
  
  $block = db_select('felix_block', 'cb')
          ->fields('cb', array('region', 'weight', 'hash', 'module', 'delta', 'data'))
          ->condition('cb.region', $_GET['region'])
          ->condition('cb.module', $_GET['module'])
          ->condition('cb.delta', $_GET['delta'])
          ->condition('cb.hash', $_GET['hash'])
          ->execute()
          ->fetchObject();
  if (!$block) {
    return MENU_NOT_FOUND;
  }
  
  $block->data = unserialize($block->data);
  
  $token = drupal_get_token($_GET['region'] . $_GET['module'] . $_GET['delta'] . $_GET['hash']);
  if ($token !== $_GET['token']) {
    return MENU_ACCESS_DENIED;
  }
  
  switch ($action) {
    case 'move-up':
    case 'move-down':
      return _felix_block_action_move($action, $block, $_GET['path']);
    case 'attributes':
      return drupal_get_form('felix_attributes_form', $region, $block, $_GET['path']);
    case 'remove':
      return drupal_get_form('felix_remove_form', $region, $block, $_GET['path']);
  }
}

function _felix_block_action_move($action, $block, $path) {
  $candidate = db_select('felix_block', 'cb')
          ->fields('cb', array('region', 'hash', 'module', 'delta', 'weight'))
          ->condition('cb.region', $block->region)
          ->condition('cb.hash', $block->hash)
          ->condition('cb.weight', $block->weight, $action == 'move-up' ? '<' : '>')
          ->orderBy('cb.weight', $action == 'move-up' ? 'DESC' : 'ASC')
          ->range(0, 1)
          ->execute()
          ->fetchObject();
  if ($candidate) {
    // Swap the block with the candidate.
    db_update('felix_block')
            ->fields(array('weight' => $candidate->weight))
            ->condition('region', $block->region)
            ->condition('hash', $block->hash)
            ->condition('module', $block->module)
            ->condition('delta', $block->delta)
            ->execute();
    db_update('felix_block')
            ->fields(array('weight' => $block->weight))
            ->condition('region', $candidate->region)
            ->condition('hash', $candidate->hash)
            ->condition('module', $candidate->module)
            ->condition('delta', $candidate->delta)
            ->execute();
  }
  else {
    drupal_set_message(t('Unable to move the block.'), 'warning');
  }
  drupal_goto($path);
}

function felix_attributes_form($form, &$form_state, $region, $block, $path) {
  $form['#felix_region'] = $region;
  $form['#felix_block'] = $block;
  $form['#felix_path'] = $path;
  
  $form['subject'] = array(
    '#type' => 'textfield',
    '#title' => t('Subject'),
    '#default_value' => isset($block->data['subject']) ? $block->data['subject'] : '',
    '#description' => t('Leave empty to use the default subject.'),
  );
  
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
    '#weight' => 100,
  );
  
  return $form;
}

function felix_attributes_form_submit($form, &$form_state) {
  $block = $form['#felix_block'];
  
  $block->data['subject'] = $form_state['values']['subject'];
  
  foreach (module_implements('felix_block_update') as $module) {
    $hook = "{$module}_felix_block_update";
    $hook($block, $form_state['values']);
  }
  
  $block->data = serialize($block->data);
  drupal_write_record('felix_block', $block, array('region', 'hash', 'module', 'delta'));
  
  // Expire external cache.
  if (module_exists('expire_cache_derivative')) {
    expire_cache_derivative(array($form['#felix_path']));
  }
  
  drupal_set_message(t('The block settings have been saved succesfully.'));
  drupal_goto($form['#felix_path']);
}

function felix_remove_form($form, &$form_state, $region, $block, $path) {
  $block = _felix_render_block($block);
  
  $form['#felix_region'] = $region;
  $form['#felix_block'] = $block;
  $form['#felix_path'] = $path;
  
  $form['info'] = array(
    '#markup' => '<p>' . t('Are you sure you want to delete the block %subject from %region?', array('%subject' => $block->subject, '%region' => $region->title)) . '</p>',
  );
  
  if ($block->module == 'node') {
    if (($node = node_load($block->delta)) && node_access('delete', $node)) {
      $form['delete_node'] = array(
        '#type' => 'checkbox',
        '#title' => t('Also delete the node'),
      );
    }
  }
  
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Remove'),
  );
  
  $form['cancel'] = array(
    '#markup' => l(t('cancel'), $path),
  );
  
  return $form;
}

function felix_remove_form_submit($form, &$form_state) {
  $region = $form['#felix_region'];
  $block = $form['#felix_block'];
  $path = $form['#felix_path'];
  
  db_delete('felix_block')
    ->condition('region', $region->name)
    ->condition('hash', $block->hash)
    ->condition('module', $block->module)
    ->condition('delta', $block->delta)
    ->execute();
  
  foreach (module_implements('felix_block_delete') as $module) {
    $hook = "{$module}_felix_block_delete";
    $hook($block);
  }
  
  if ($block->module == 'node' && !empty($form_state['values']['delete_node'])) {
    node_delete($block->delta);
  }
  
  // Expire external cache.
  if (module_exists('expire_cache_derivative')) {
    expire_cache_derivative(array($path));
  }
  
  drupal_set_message(t('The block %subject has been deleted from region %region.', array('%subject' => $block->subject, '%region' => $region->title)));
  drupal_goto($path);
}
